Attribute VB_Name = "RodRego"
'**********************************************************
'*** Global module for the RodRego application.  **********
'**********************************************************
'***     Programmer : Nikolai Shvertner                 ***
'***     Date       : 6/20/2001                         ***
'***     Compiler   : Visual Basic 5                    ***
'***     Notes      :                                   ***
'***     Revisions  : 9/20/2001                         ***
'**********************************************************
Option Explicit

' Data structure for one full RodRego instruction
Type Instruction
    OpCode As Byte    ' Code of the operator
    Register As Byte  ' The register to which the operation is applied
    Line1 As Byte     ' Next line to go to
    Line2 As Byte     ' Alternative line to go to
End Type

' A pair to be used in function returns
Type Duce
    first As Byte
    second As Byte
End Type

' The contents of the registers go here
Public Register(0 To 9) As Integer

' The parsed program is encoded here
Public Program(0 To 255) As Instruction

' Here we store the indexes of the valid entries
' in the Program array
Public ProgPointers(0 To 255) As Integer
Public lenProgPointers As Integer ' The number of elements in ProgPointers

' A flag to indicate that the text boxes in RodRego
' has been touched
Public Dirty As Boolean

' Variables to be used when scrolling around the LCD display
Public Deviation, max_dev As Integer

' The enabled status for button "Step"
Public btnStep_Enabled As Boolean

' Delay in milliseconds while interpreting a program
Public rod_interval As Integer

' Constants defining the language and the operator codes
Global Const OPCODE_NONE = 0
Global Const OPCODE_INC = 1
Global Const OPCODE_DEB = 2
Global Const OPCODE_END = 3
Global Const INSTR_NONE = "NONE"
Global Const INSTR_INC = "INC"
Global Const INSTR_DEB = "DEB"
Global Const INSTR_END = "END"

' Color constants - do not alter their values or the
' program might not function properly!
Global Const COLOR_RED = 0
Global Const COLOR_GREEN = 1

Global Const SND_INCREMENT = 0
Global Const SND_DECREMENT = 1
Global Const SND_DECREMENT2 = 2
Global Const SND_FINISH = 3

Global Const conInterval = 50
Global Const conIntervalPlus = 55


Sub EditCopyProc()
    ' Copy the selected text onto the Clipboard.
    Clipboard.SetText frmMain.txtProg.SelText
End Sub

Sub EditCutProc()
    ' Copy the selected text onto the Clipboard.
    Clipboard.SetText frmMain.txtProg.SelText
    ' Delete the selected text.
    frmMain.txtProg.SelText = ""
End Sub

Sub EditPasteProc()
    ' Place the text from the Clipboard into the active control.
    frmMain.txtProg.SelText = Clipboard.GetText()
End Sub

Sub FileNew()
    ' If the file has changed, save it
    If Dirty = True Then
        If FileSave = False Then Exit Sub
    End If
    ' Clear the textbox and update the caption.
    frmMain.txtProg.Text = ""
    frmMain.Caption = "RodRego - Untitled"
End Sub

Function FileSave() As Boolean
    Dim strFilename As String

    If frmMain.Caption = "RodRego - Untitled" Then
        ' The file hasn't been saved yet.
        ' Get the filename, and then call the save procedure, GetFileName.
        strFilename = GetFileName(strFilename)
    Else
        ' The form's Caption contains the name of the open file.
        strFilename = Right(frmMain.Caption, Len(frmMain.Caption) - 10)
    End If
    ' Call the save procedure. If Filename = Empty, then
    ' the user chose Cancel in the Save As dialog box; otherwise,
    ' save the file.
    If strFilename <> "" Then
        SaveFileAs strFilename
        FileSave = True
    Else
        FileSave = False
    End If
End Function

Sub GoLineProc(linenum As Integer)
    ' Select a line in the text box in RodRego
    
    Dim i, k As Integer
    Dim ch As String
    
    i = 1 ' Marker of the line start position
    
    ' If the line is non-positive, leave
    If (linenum < 1) Then Exit Sub
      
    ' How many lines are left to the requested line?
    While (linenum > 1)
    
        ' Advance the marker until reaching the end of the line
        ch = Mid(frmMain.txtProg.Text, i, 1)
        While (ch <> Chr(13))
            i = i + 1
            If (i > Len(frmMain.txtProg.Text)) Then
                MsgBox ("Line out of range")
                Exit Sub
            End If
            ch = Mid(frmMain.txtProg.Text, i, 1)
        Wend
        
        ' If possible, continue the operation
        i = i + 1
        If (i > Len(frmMain.txtProg.Text)) Then
            MsgBox ("Line out of range")
            Exit Sub
        End If
        linenum = linenum - 1
    Wend
    
    ' Now we have the starting position of the line to be
    ' selected. Let's find the length of the selection.
    k = 0
    
    Do
        ch = Mid(frmMain.txtProg.Text, i + k, 1)
        k = k + 1
        If (i + k > Len(frmMain.txtProg.Text)) Then ch = Chr(13)
    Loop While (ch <> Chr(13))

    ' Perform the selecting
    frmMain.txtProg.SelStart = i - 1
    frmMain.txtProg.SelLength = k
End Sub

Sub drawProgram(Deviation As Integer)
    ' Display the program on the LCD display
    
    Dim end_index, i, dev_i As Integer
    
    ' Determine the final member of the program pointers
    ' array to be displayed
    If (lenProgPointers > 12) Then
        end_index = 11
    Else
        end_index = lenProgPointers - 1
    End If

    ' Clear all lines
    For i = 0 To 11
        clearLine i
    Next i

    ' Calculate the exact indexes of the Program
    ' statements to be displayed and draw them in red
    For i = 0 To end_index
      dev_i = i + Deviation
      drawLine Program(ProgPointers(dev_i)), i, COLOR_RED
    Next i
End Sub

Sub initializeDrawProgram()
    ' Set all required variables when initially drawing a
    ' program on the LCD display
    Dim i As Integer

    Deviation = 0 ' Start at the top of the program

    ' Set the required Scroll button settings
    If lenProgPointers > 12 Then
        max_dev = lenProgPointers - 12
        frmMain.btnScrollUp.Enabled = False
        frmMain.btnScrollDown.Enabled = True
    Else
        frmMain.btnScrollUp.Enabled = False
        frmMain.btnScrollDown.Enabled = False
    End If

    ' Draw the program with the required deviation
    drawProgram (Deviation)
End Sub

Sub select_line(line As Integer) ' "line" is a line in the program
    ' Highlight a line on the LCD display by redrawing it in red
    Dim i, pos As Integer
    
    pos = 0
    
    ' Determine the relative position of the requested program
    ' line in the list of lines to be displayed on the LCD
    ' screen
    While ProgPointers(pos) <> line And pos < lenProgPointers
        pos = pos + 1
    Wend
    
    ' Adjust the view as required
    While pos < Deviation
        frmMain.btnScrollUp_Click
    Wend
    
    While pos > 11 + Deviation
        frmMain.btnScrollDown_Click
    Wend
    
    ' Redraw all lines in red or clear them as necessary
    ' *** NOTE: This needs to be optimized in the next version
    For i = 0 To 11
        If i + Deviation < lenProgPointers Then
            drawLine Program(ProgPointers(i)), i, COLOR_RED
        Else
            clearLine i
        End If
    Next i

    ' Redraw the required line in green
    drawLine Program(ProgPointers(pos)), pos - Deviation, COLOR_GREEN
End Sub

Function loadProgram()
    ' Parse a program into memory
    Dim err As error
    err = ParseString(frmMain.txtProg.Text) ' call the parser
    
    loadProgram = False
    
    If err.error_code <> SUCCESS Then
        ' The parse failed. Go to the problematic
        ' line and display an error message.
        GoLineProc (err.line + 1)
        frmMain.txtProg.SetFocus
        frmMain.Status.SimpleText = "(Line " & (err.line + 1) & ") Error: " & getError(err.error_code)
    Else
        ' Do a level 2 check of the program syntax
        err = VerifyProg(frmMain.txtProg.Text)
        If (err.error_code <> SUCCESS) Then
            ' Still problems
            frmMain.Status.SimpleText = "Error: " & getError(err.error_code)
        Else
            ' Everything succeeded
            loadProgram = True
        End If
    End If
End Function

Sub drawLine(inst As Instruction, ByVal line As Byte, color As Byte)
    ' Draws a line on the LCD display
    '   + inst  = the instruction to be displayed
    '   + line  = the index at which the line should be drawn
    '   + color = the color to be used (COLOR_RED or COLOR_GREEN)
    Dim tmp_d As Duce
    
    ' Obtain the digit of the real "line number"
    tmp_d = breakDigits(ProgPointers(line + Deviation))
    
    ' Set the pictures of the line number to the required images
    If tmp_d.first > 0 Then
        frmMain.imgLine0(line).Picture = frmMain.imgLCD(tmp_d.first + 10 * color).Picture
    Else
        frmMain.imgLine0(line).Picture = frmMain.imgNone.Picture
    End If
    frmMain.imgLine1(line).Picture = frmMain.imgLCD(tmp_d.second + 10 * color).Picture
    
    ' Check the instruction
    Select Case (inst.OpCode)
        Case OPCODE_END
            ' END operator has no arguments
            frmMain.imgOp(line).Picture = frmMain.imgEND(color).Picture
            
        Case OPCODE_INC
            ' INC has 2 arguments
            frmMain.imgOp(line).Picture = frmMain.imgINC(color).Picture
                
            tmp_d = breakDigits(inst.Register)
            frmMain.imgReg(line).Picture = frmMain.imgLCD(tmp_d.second + 10 * color).Picture
   
            tmp_d = breakDigits(inst.Line1)
            If tmp_d.first > 0 Then
                frmMain.imgLP0(line).Picture = frmMain.imgLCD(tmp_d.first + 10 * color).Picture
            Else
                frmMain.imgLP0(line).Picture = frmMain.imgNone.Picture
            End If
            frmMain.imgLP1(line).Picture = frmMain.imgLCD(tmp_d.second + 10 * color).Picture
            
        Case OPCODE_DEB
            ' DEB has all 3 arguments
            frmMain.imgOp(line).Picture = frmMain.imgDEB(color).Picture
            
            tmp_d = breakDigits(inst.Register)
            frmMain.imgReg(line).Picture = frmMain.imgLCD(tmp_d.second + 10 * color).Picture
   
            tmp_d = breakDigits(inst.Line1)
            If tmp_d.first > 0 Then
                frmMain.imgLP0(line).Picture = frmMain.imgLCD(tmp_d.first + 10 * color).Picture
            Else
                frmMain.imgLP0(line).Picture = frmMain.imgNone.Picture
            End If
            frmMain.imgLP1(line).Picture = frmMain.imgLCD(tmp_d.second + 10 * color).Picture

            tmp_d = breakDigits(inst.Line2)
            If tmp_d.first > 0 Then
                frmMain.imgLN0(line).Picture = frmMain.imgLCD(tmp_d.first + 10 * color).Picture
            Else
                frmMain.imgLN0(line).Picture = frmMain.imgNone.Picture
            End If
            frmMain.imgLN1(line).Picture = frmMain.imgLCD(tmp_d.second + 10 * color).Picture
            
        Case Else
    End Select
End Sub

Sub clearLine(ByVal line As Byte)
    ' Clear a line on the LCD display by placing the
    ' space-holder images
    frmMain.imgLine0(line).Picture = frmMain.imgNone.Picture
    frmMain.imgLine1(line).Picture = frmMain.imgNone.Picture
    frmMain.imgOp(line).Picture = frmMain.imgNoOp.Picture
    frmMain.imgReg(line).Picture = frmMain.imgNone.Picture
    frmMain.imgLP0(line).Picture = frmMain.imgNone.Picture
    frmMain.imgLP1(line).Picture = frmMain.imgNone.Picture
    frmMain.imgLN0(line).Picture = frmMain.imgNone.Picture
    frmMain.imgLN1(line).Picture = frmMain.imgNone.Picture
End Sub

Function breakDigits(ByVal num As Byte) As Duce
    ' Return the two rightmost digits of a number
    breakDigits.first = (Int(num / 10)) Mod 10
    breakDigits.second = num Mod 10
End Function

Sub Play_Sound(sound_id As Integer)
    ' Play one of the sound effects
    With frmMain.WAVCtrl(sound_id)
        .Command = "stop"
        .From = 0
        .UpdateInterval = conInterval
        If frmMain.mnuOptionsSound.Checked Then .Command = "play"
    End With
End Sub
